@@ -203,7 +203,7 @@ def do_sync(
         clean=clean,
         dry_run=dry_run,
         no_editable=no_editable,
-        install_self=not no_self and "default" in selection and bool(project.name),
+        install_self=not no_self and bool(project.name),
         reinstall=reinstall,
         only_keep=only_keep,
         fail_fast=fail_fast,
