@@ -78,12 +78,11 @@ class Core:
             "--config",
             help="Specify another config file path [env var: PDM_CONFIG_FILE] ",
         )
-        self.parser._positionals.title = "Commands"
         verbose_option.add_to_parser(self.parser)
         ignore_python_option.add_to_parser(self.parser)
         pep582_option.add_to_parser(self.parser)
 
-        self.subparsers = self.parser.add_subparsers(parser_class=ArgumentParser, metavar="__root__")
+        self.subparsers = self.parser.add_subparsers(parser_class=ArgumentParser, title="commands", metavar="")
         for _, name, _ in pkgutil.iter_modules(COMMANDS_MODULE_PATH):
             module = importlib.import_module(f"pdm.cli.commands.{name}", __name__)
             try:
@@ -153,7 +152,8 @@ class Core:
             use_venv(project, cast(str, options.use_venv))
 
         if command is None:
-            self.parser.error("No command given")
+            self.parser.print_help()
+            sys.exit(0)
         command.handle(project, options)
 
     def _get_cli_args(self, args: list[str], obj: Project | None) -> list[str]:
@@ -176,8 +176,9 @@ class Core:
     ) -> None:
         """The main entry function"""
 
-        # Ensure same behavior while testing and using the CLI
-        args = self._get_cli_args(args or sys.argv[1:], obj)
+        if args is None:
+            args = []
+        args = self._get_cli_args(args, obj)
         # Keep it for after project parsing to check if its a defined script
         root_script = None
         try:
@@ -286,4 +287,4 @@ class Core:
 
 def main(args: list[str] | None = None) -> None:
     """The CLI entry function"""
-    return Core().main(args)
+    return Core().main(args or sys.argv[1:])
